// SPDX-FileCopyrightText: 2022 Matteo Settenvini <matteo.settenvini@montecristosoftware.eu>
// SPDX-License-Identifier: AGPL-3.0-or-later

pub fn ical_escape_text(text: &str) -> String {
    /* https://www.kanzaki.com/docs/ical/text.html

    The "TEXT" property values may also contain special characters that
    are used to signify delimiters, such as a COMMA character for lists of
    values or a SEMICOLON character for structured values. In order to
    support the inclusion of these special characters in "TEXT" property
    values, they MUST be escaped with a BACKSLASH character. A BACKSLASH
    character (US-ASCII decimal 92) in a "TEXT" property value MUST be
    escaped with another BACKSLASH character. A COMMA character in a "TEXT"
    property value MUST be escaped with a BACKSLASH character
    (US-ASCII decimal 92). A SEMICOLON character in a "TEXT" property
    value MUST be escaped with a BACKSLASH character (US-ASCII decimal
    92). However, a COLON character in a "TEXT" property value SHALL
    NOT be escaped with a BACKSLASH character.
    */

    let mut out = Vec::<u8>::with_capacity(text.len());
    for c in text.as_bytes() {
        match c {
            b'\\' => out.extend_from_slice(&[b'\\', b'\\']),
            b',' => out.extend_from_slice(&[b'\\', b',']),
            b';' => out.extend_from_slice(&[b'\\', b';']),
            _ => out.push(*c),
        }
    }

    unsafe { String::from_utf8_unchecked(out) }
}
